##############################################################################
#
# Copyright (c) 2003 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Interfaces for standard python exceptions
"""
from zope.interface import Interface
from zope.interface import classImplements

class IException(Interface):
    "Interface for `Exception`"
classImplements(Exception, IException)


class IStandardError(IException):
    "Interface for `StandardError` (Python 2 only.)"
try:
    classImplements(StandardError, IStandardError)
except NameError:  #pragma NO COVER
    pass # StandardError does not exist in Python 3


class IWarning(IException):
    "Interface for `Warning`"
classImplements(Warning, IWarning)


class ISyntaxError(IStandardError):
    "Interface for `SyntaxError`"
classImplements(SyntaxError, ISyntaxError)


class ILookupError(IStandardError):
    "Interface for `LookupError`"
classImplements(LookupError, ILookupError)


class IValueError(IStandardError):
    "Interface for `ValueError`"
classImplements(ValueError, IValueError)


class IRuntimeError(IStandardError):
    "Interface for `RuntimeError`"
classImplements(RuntimeError, IRuntimeError)


class IArithmeticError(IStandardError):
    "Interface for `ArithmeticError`"
classImplements(ArithmeticError, IArithmeticError)


class IAssertionError(IStandardError):
    "Interface for `AssertionError`"
classImplements(AssertionError, IAssertionError)


class IAttributeError(IStandardError):
    "Interface for `AttributeError`"
classImplements(AttributeError, IAttributeError)


class IDeprecationWarning(IWarning):
    "Interface for `DeprecationWarning`"
classImplements(DeprecationWarning, IDeprecationWarning)


class IEOFError(IStandardError):
    "Interface for `EOFError`"
classImplements(EOFError, IEOFError)


class IEnvironmentError(IStandardError):
    "Interface for `EnvironmentError`"
classImplements(EnvironmentError, IEnvironmentError)


class IFloatingPointError(IArithmeticError):
    "Interface for `FloatingPointError`"
classImplements(FloatingPointError, IFloatingPointError)


class IIOError(IEnvironmentError):
    "Interface for `IOError`"
classImplements(IOError, IIOError)


class IImportError(IStandardError):
    "Interface for `ImportError`"
classImplements(ImportError, IImportError)


class IIndentationError(ISyntaxError):
    "Interface for `IndentationError`"
classImplements(IndentationError, IIndentationError)


class IIndexError(ILookupError):
    "Interface for `IndexError`"
classImplements(IndexError, IIndexError)


class IKeyError(ILookupError):
    "Interface for `KeyError`"
classImplements(KeyError, IKeyError)


class IKeyboardInterrupt(IStandardError):
    "Interface for `KeyboardInterrupt`"
classImplements(KeyboardInterrupt, IKeyboardInterrupt)


class IMemoryError(IStandardError):
    "Interface for `MemoryError`"
classImplements(MemoryError, IMemoryError)


class INameError(IStandardError):
    "Interface for `NameError`"
classImplements(NameError, INameError)


class INotImplementedError(IRuntimeError):
    "Interface for `NotImplementedError`"
classImplements(NotImplementedError, INotImplementedError)


class IOSError(IEnvironmentError):
    "Interface for `OSError`"
classImplements(OSError, IOSError)


class IOverflowError(IArithmeticError):
    "Interface for `ArithmeticError`"
classImplements(OverflowError, IOverflowError)


class IOverflowWarning(IWarning):
    """Deprecated, no standard class implements this.

    This was the interface for ``OverflowWarning`` prior to Python 2.5,
    but that class was removed for all versions after that.
    """


class IReferenceError(IStandardError):
    "Interface for `ReferenceError`"
classImplements(ReferenceError, IReferenceError)


class IRuntimeWarning(IWarning):
    "Interface for `RuntimeWarning`"
classImplements(RuntimeWarning, IRuntimeWarning)


class IStopIteration(IException):
    "Interface for `StopIteration`"
classImplements(StopIteration, IStopIteration)


class ISyntaxWarning(IWarning):
    "Interface for `SyntaxWarning`"
classImplements(SyntaxWarning, ISyntaxWarning)


class ISystemError(IStandardError):
    "Interface for `SystemError`"
classImplements(SystemError, ISystemError)


class ISystemExit(IException):
    "Interface for `SystemExit`"
classImplements(SystemExit, ISystemExit)


class ITabError(IIndentationError):
    "Interface for `TabError`"
classImplements(TabError, ITabError)


class ITypeError(IStandardError):
    "Interface for `TypeError`"
classImplements(TypeError, ITypeError)


class IUnboundLocalError(INameError):
    "Interface for `UnboundLocalError`"
classImplements(UnboundLocalError, IUnboundLocalError)


class IUnicodeError(IValueError):
    "Interface for `UnicodeError`"
classImplements(UnicodeError, IUnicodeError)


class IUserWarning(IWarning):
    "Interface for `UserWarning`"
classImplements(UserWarning, IUserWarning)


class IZeroDivisionError(IArithmeticError):
    "Interface for `ZeroDivisionError`"
classImplements(ZeroDivisionError, IZeroDivisionError)



##############################################################################
#
# Copyright (c) 2006 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
Support functions for dealing with differences in platforms, including Python
versions and implementations.

This file should have no imports from the rest of zope.interface because it is
used during early bootstrapping.
"""
import os
import sys
import types

if sys.version_info[0] < 3:

    def _normalize_name(name):
        if isinstance(name, basestring):
            return unicode(name)
        raise TypeError("name must be a regular or unicode string")

    CLASS_TYPES = (type, types.ClassType)
    STRING_TYPES = (basestring,)

    _BUILTINS = '__builtin__'

    PYTHON3 = False
    PYTHON2 = True

else:

    def _normalize_name(name):
        if isinstance(name, bytes):
            name = str(name, 'ascii')
        if isinstance(name, str):
            return name
        raise TypeError("name must be a string or ASCII-only bytes")

    CLASS_TYPES = (type,)
    STRING_TYPES = (str,)

    _BUILTINS = 'builtins'

    PYTHON3 = True
    PYTHON2 = False

PYPY = hasattr(sys, 'pypy_version_info')
PYPY2 = PYTHON2 and PYPY

def _skip_under_py3k(test_method):
    import unittest
    return unittest.skipIf(sys.version_info[0] >= 3, "Only on Python 2")(test_method)


def _skip_under_py2(test_method):
    import unittest
    return unittest.skipIf(sys.version_info[0] < 3, "Only on Python 3")(test_method)


def _c_optimizations_required():
    """
    Return a true value if the C optimizations are required.

    This uses the ``PURE_PYTHON`` variable as documented in `_use_c_impl`.
    """
    pure_env = os.environ.get('PURE_PYTHON')
    require_c = pure_env == "0"
    return require_c


def _c_optimizations_available():
    """
    Return the C optimization module, if available, otherwise
    a false value.

    If the optimizations are required but not available, this
    raises the ImportError.

    This does not say whether they should be used or not.
    """
    catch = () if _c_optimizations_required() else (ImportError,)
    try:
        from zope.interface import _zope_interface_coptimizations as c_opt
        return c_opt
    except catch: # pragma: no cover (only Jython doesn't build extensions)
        return False


def _c_optimizations_ignored():
    """
    The opposite of `_c_optimizations_required`.
    """
    pure_env = os.environ.get('PURE_PYTHON')
    return pure_env is not None and pure_env != "0"


def _should_attempt_c_optimizations():
    """
    Return a true value if we should attempt to use the C optimizations.

    This takes into account whether we're on PyPy and the value of the
    ``PURE_PYTHON`` environment variable, as defined in `_use_c_impl`.
    """
    is_pypy = hasattr(sys, 'pypy_version_info')

    if _c_optimizations_required():
        return True
    if is_pypy:
        return False
    return not _c_optimizations_ignored()


def _use_c_impl(py_impl, name=None, globs=None):
    """
    Decorator. Given an object implemented in Python, with a name like
    ``Foo``, import the corresponding C implementation from
    ``zope.interface._zope_interface_coptimizations`` with the name
    ``Foo`` and use it instead.

    If the ``PURE_PYTHON`` environment variable is set to any value
    other than ``"0"``, or we're on PyPy, ignore the C implementation
    and return the Python version. If the C implementation cannot be
    imported, return the Python version. If ``PURE_PYTHON`` is set to
    0, *require* the C implementation (let the ImportError propagate);
    note that PyPy can import the C implementation in this case (and all
    tests pass).

    In all cases, the Python version is kept available. in the module
    globals with the name ``FooPy`` and the name ``FooFallback`` (both
    conventions have been used; the C implementation of some functions
    looks for the ``Fallback`` version, as do some of the Sphinx
    documents).

    Example::

        @_use_c_impl
        class Foo(object):
            ...
    """
    name = name or py_impl.__name__
    globs = globs or sys._getframe(1).f_globals

    def find_impl():
        if not _should_attempt_c_optimizations():
            return py_impl

        c_opt = _c_optimizations_available()
        if not c_opt: # pragma: no cover (only Jython doesn't build extensions)
            return py_impl

        __traceback_info__ = c_opt
        return getattr(c_opt, name)

    c_impl = find_impl()
    # Always make available by the FooPy name and FooFallback
    # name (for testing and documentation)
    globs[name + 'Py'] = py_impl
    globs[name + 'Fallback'] = py_impl

    return c_impl



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Interfaces

This package implements the Python "scarecrow" proposal.

The package exports two objects, `Interface` and `Attribute` directly. It also
exports several helper methods. Interface is used to create an interface with
a class statement, as in:

  class IMyInterface(Interface):
    '''Interface documentation
    '''

    def meth(arg1, arg2):
        '''Documentation for meth
        '''

    # Note that there is no self argument

To find out what you can do with interfaces, see the interface
interface, `IInterface` in the `interfaces` module.

The package has several public modules:

  o `declarations` provides utilities to declare interfaces on objects. It
    also provides a wide range of helpful utilities that aid in managing
    declared interfaces. Most of its public names are however imported here.

  o `document` has a utility for documenting an interface as structured text.

  o `exceptions` has the interface-defined exceptions

  o `interfaces` contains a list of all public interfaces for this package.

  o `verify` has utilities for verifying implementations of interfaces.

See the module doc strings for more information.
"""
__docformat__ = 'restructuredtext'
# pylint:disable=wrong-import-position,unused-import
from zope.interface.interface import Interface
from zope.interface.interface import _wire

# Need to actually get the interface elements to implement the right interfaces
_wire()
del _wire

from zope.interface.declarations import Declaration
from zope.interface.declarations import alsoProvides
from zope.interface.declarations import classImplements
from zope.interface.declarations import classImplementsFirst
from zope.interface.declarations import classImplementsOnly
from zope.interface.declarations import classProvides
from zope.interface.declarations import directlyProvidedBy
from zope.interface.declarations import directlyProvides
from zope.interface.declarations import implementedBy
from zope.interface.declarations import implementer
from zope.interface.declarations import implementer_only
from zope.interface.declarations import implements
from zope.interface.declarations import implementsOnly
from zope.interface.declarations import moduleProvides
from zope.interface.declarations import named
from zope.interface.declarations import noLongerProvides
from zope.interface.declarations import providedBy
from zope.interface.declarations import provider

from zope.interface.exceptions import Invalid

from zope.interface.interface import Attribute
from zope.interface.interface import interfacemethod
from zope.interface.interface import invariant
from zope.interface.interface import taggedValue

# The following are to make spec pickles cleaner
from zope.interface.declarations import Provides


from zope.interface.interfaces import IInterfaceDeclaration

moduleProvides(IInterfaceDeclaration)

__all__ = ('Interface', 'Attribute') + tuple(IInterfaceDeclaration)

assert all(k in globals() for k in __all__)
